@@ -54,11 +54,11 @@ sub import {
     return;
 }
 
-die "Perl lib version (5.14.2) doesn't match executable '$0' version ($])"
+die "Perl lib version (5.16.0) doesn't match executable '$0' version ($])"
     unless $^V;
 
-$^V eq 5.14.2
-    or die "Perl lib version (5.14.2) doesn't match executable '$0' version (" .
+$^V eq 5.16.0
+    or die "Perl lib version (5.16.0) doesn't match executable '$0' version (" .
 	sprintf("v%vd",$^V) . ")";
 
 
@@ -83,7 +83,7 @@ sub AUTOLOAD {
 
 # tie returns the object, so the value returned to require will be true.
 tie %Config, 'Config', {
-    archlibexp => '/usr/lib/perl5/5.14.2/x86_64-linux-thread-multi',
+    archlibexp => '/usr/lib/perl5/5.16.0/x86_64-linux-thread-multi',
     archname => 'x86_64-linux-thread-multi',
     cc => 'cc',
     d_readlink => 'define',
@@ -97,14 +97,14 @@ tie %Config, 'Config', {
     ldlibpthname => 'LD_LIBRARY_PATH',
     libpth => '/lib64 /usr/lib64 /usr/local/lib64',
     osname => 'linux',
-    osvers => '3.4.11-2.16-xen',
+    osvers => '3.4.6-2.10-default',
     path_sep => ':',
-    privlibexp => '/usr/lib/perl5/5.14.2',
+    privlibexp => '/usr/lib/perl5/5.16.0',
     scriptdir => '/usr/bin',
-    sitearchexp => '/usr/lib/perl5/site_perl/5.14.2/x86_64-linux-thread-multi',
-    sitelibexp => '/usr/lib/perl5/site_perl/5.14.2',
+    sitearchexp => '/usr/lib/perl5/site_perl/5.16.0/x86_64-linux-thread-multi',
+    sitelibexp => '/usr/lib/perl5/site_perl/5.16.0',
     so => 'so',
     useithreads => 'define',
     usevendorprefix => 'define',
-    version => '5.14.2',
+    version => '5.16.0',
 };
