@@ -18,7 +18,7 @@
 #   Purpose:       Call a perl script within a YCP script
 #
 #----------------------------------------------------------------------
-# $Id: ycp.pm 53219 2008-11-12 15:01:55Z schubi2 $
+# $Id$
 
 package ycp;
 
@@ -79,9 +79,6 @@ use strict;
 use Time::localtime;
 use Sys::Hostname;
 
-# needed for printing character (>255)
-binmode STDOUT, ':utf8';
-
 @ISA     = qw(Exporter);
 
 
@@ -718,6 +715,13 @@ sub Return ($;$)
 sub WriteYcpString ($)
 {
     my $string = shift;
+    # We deal with binary strings (binmode raw), and most code is fine with it.
+    # But XML parsers will mark the strings as text, so convert them back.
+    # bnc#512536, thanks mls
+    # (Note that binmode STDOUT, :utf8 without the same for STDIN would
+    # make bnc#448217).
+    utf8::encode($string) if utf8::is_utf8($string);
+
     my @substrings = split /\\/, $string, -1;
     foreach my $substring (@substrings)
     {
