@@ -120,18 +120,24 @@ sub ZoneWrite {
 	return 0;
     }
 
-    if (! ($zone_map{"modified"} || $save_all))
-    {
-	y2milestone ("Skipping zone $zone_name, wasn't modified");
-	return 1;
-    }
-
     if ($zone_name eq "localhost" || $zone_name eq "0.0.127.in-addr.arpa")
     {
 	y2milestone ("Skipping system zone $zone_name");
 	return 1;
     }
 
+    if ($save_all) {
+	y2milestone ("Saving all zones, flagging ".$zone_name." zone as modified");
+	$zone_map{"modified"} = 1;
+    }
+
+    # zone not modified (it's not modified if the flag is not set)
+    if (! (defined $zone_map{"modified"} ? $zone_map{"modified"} : 0))
+    {
+	y2milestone ("Skipping zone $zone_name, wasn't modified");
+	return 1;
+    }
+
     my $zone_file = $zone_map{"file"} || "";
     # creating temporary zone file if zone is slave
     if ($zone_file eq "" && defined $zone_map{"type"} && $zone_map{"type"} eq "slave") {
@@ -526,6 +532,8 @@ sub StoreZone {
 	$zones[$current_zone_index] = \%tmp_current_zone;
     }
 
+    $self->SetModified ();
+
     return 1;
 }
 
@@ -645,12 +653,20 @@ sub GetStartService {
     return $start_service;
 }
 
+sub UseLdapModified {
+    my $self = shift;
+
+    $self->SetModified ();
+    # Save all zones as they are into the new config location (LDAP/files)
+    $save_all = 1;
+}
+
 BEGIN { $TYPEINFO{SetUseLdap} = [ "function", "boolean", "boolean" ];}
 sub SetUseLdap {
     my $self = shift;
-    $use_ldap = shift;
+    my $new_use_ldap = shift;
 
-    if ($use_ldap) {
+    if ($new_use_ldap) {
 	# trying init LDAP if use_ldap selected
 	my $success = $self->LdapInit (1, 1);
 
@@ -659,9 +675,10 @@ sub SetUseLdap {
 	}
     }
 
-    $self->SetModified ();
+    $use_ldap = $new_use_ldap;
+    y2milestone ("Using LDAP set to: ".$use_ldap);
 
-    $save_all = 1;
+    $self->UseLdapModified();
 
     return 1;
 }
@@ -894,6 +911,11 @@ BEGIN { $TYPEINFO{GetConfigurationStat} = ["function", "string"]; }
 sub GetConfigurationStat {
     my $class = shift;
 
+    unless (FileUtils->Exists ($configfile)) {
+        y2error('Cannot stat file '.$configfile.', file does not exist');
+        return undef;
+    }
+
     my $ret = SCR->Execute (".target.bash_output",
 	"stat --format='rights: %a, blocks: %b, size: %s, owner: %u:%g changed: %Z, modifyied: %Y' ".$configfile
     );
@@ -972,7 +994,11 @@ sub Read {
 
     Progress->NextStage ();
 
-    y2milestone("Converting configfile: ", SCR->Execute (".dns.named_conf_convert", $configfile));
+    if (FileUtils->Exists ($configfile)) {
+        y2milestone('Converting configfile: ', SCR->Execute ('.dns.named_conf_convert', $configfile));
+    } else {
+        y2error('Config file '.$configfile.' does not exist, file cannot be converted');
+    }
 
     $configuration_timestamp = $self->GetConfigurationStat();
 
@@ -1253,6 +1279,14 @@ sub Write {
 
     my $ok = 1;
 
+    foreach my $z (@zones) {
+	if (defined $z->{"modified"} and $z->{"modified"} == 1) {
+	    y2milestone ("Some zones were modified");
+	    $self->SetModified();
+	    last;
+	}
+    }
+
     if ((! $modified) && (! SuSEFirewall->GetModified()))
     {
 	return $ok;
@@ -1266,7 +1300,10 @@ sub Write {
     # authenticate to LDAP 
     if ($use_ldap)
     {
-	LdapPrepareToWrite ();
+	unless (LdapPrepareToWrite ()) {
+	    y2error ("Cannot write to LDAP");
+	    return 0;
+	}
     }
 
     foreach my $z (@zones) {
@@ -1543,13 +1580,13 @@ sub Summary {
     if ($start_service)
     {
 	# summary string
-	push (@ret, __("The DNS server starts when booting the system"));
+	push (@ret, __("The DNS server starts when booting the system."));
     }
     else
     {
 	push (@ret,
 	    # summary string
-	    __("The DNS server does not start when booting the system"));
+	    __("The DNS server does not start when booting the system."));
     }
 
     my @zones_descr = map {
@@ -1674,7 +1711,8 @@ sub LdapInit {
 	    if ($ask_user_to_enable_ldap) {
 		# yes-no popup
 		$use_ldap = Popup->YesNo (__("Enable LDAP support?"));
-		y2milestone ("User choose to use LDAP: $use_ldap");
+		$self->UseLdapModified();
+		y2milestone ("User chose to use LDAP: $use_ldap");
 	    # just disable LDAP
 	    } else {
 		$use_ldap = 0;
@@ -1774,6 +1812,24 @@ LDAP support will not be active."));
     return;
 }
 
+# Checks whether all required schemas are correctly included
+# Gets an LDAP Domain as a parameter
+# Returns boolean
+sub LdapSchemasPresent ($) {
+    my $ldap_domain = shift;
+    my $ret = 1;
+    local $Data::Dumper::Indent = 0;
+
+    my $dns_zone = "ou=DNS,".$ldap_domain;
+    my $dz_ret = Ldap->GetLDAPEntry ($dns_zone);
+    y2milestone ("LDAP: DNS (".$dns_zone.") found: ".Dumper($dz_ret));
+
+    # Record not found
+    $ret = 0 if (! defined $dz_ret || keys(%{$dz_ret}) == 0);
+
+    return $ret;
+}
+
 BEGIN { $TYPEINFO{LdapPrepareToWrite} = ["function", "boolean"];}
 sub LdapPrepareToWrite {
     my $self = shift;
@@ -1784,12 +1840,14 @@ sub LdapPrepareToWrite {
 
     # check if the schema is properly included
     NetworkInterfaces->Read ();
-    if ($ldap_server eq "127.0.0.1" || $ldap_server eq "localhost"
+    if ($ldap_server =~ /^127\.0\.0\.[0-9]+$/ || $ldap_server eq "localhost" || $ldap_server eq "::1"
 	|| -1 != index (lc ($ldap_server), lc (Hostname->CurrentHostname()))
 	|| 0 != scalar (@{NetworkInterfaces->Locate ("IPADDR", $ldap_server)}))
     {
 	y2milestone ("LDAP server is local, checking included schemas");
-	LdapServerAccess->AddLdapSchemas(["/etc/openldap/schema/dnszone.schema"],1);
+	LdapServerAccess->AddLdapSchemas(
+	    ["/etc/openldap/schema/yast.schema", "/etc/openldap/schema/dnszone.schema"],1
+	);
     }
     else
     {
@@ -1797,7 +1855,10 @@ sub LdapPrepareToWrite {
     }
 
     # connect to the LDAP server
-    my $ret = Ldap->LDAPInit (0, 0);
+    my $ret = Ldap->LDAPInit ();
+    # BNC #679960
+    $ret = _("Unknown LDAP initialization error.") unless (defined $ret);
+
     if ($ret ne "")
     {
 	Ldap->LDAPErrorMessage ("init", $ret);
@@ -1837,6 +1898,11 @@ sub LdapPrepareToWrite {
     } 
     Ldap->SetGUI(YaST::YCP::Boolean(1)); 
 
+    # If non-local server is used, schemas are not handled but required anyway
+    # BNC #710430 and BNC #690237
+    # Currently just for logging purpose here
+    LdapSchemasPresent($ldap_domain);
+
     # find suseDnsConfiguration object
     %ldap_query = (
         "base_dn" => $ldap_config_dn,
@@ -1867,7 +1933,7 @@ sub LdapPrepareToWrite {
 	    my $err = SCR->Read (".ldap.error") || {};
 	    my $err_descr = Dumper ($err);
 	    y2error ("Error descr: $err_descr");
-	    return;
+	    return 0;
 	}
 	%found = %ldap_object;
     }
@@ -1893,7 +1959,7 @@ sub LdapPrepareToWrite {
 		my $err = SCR->Read (".ldap.error") || {};
 		my $err_descr = Dumper ($err);
 		y2error ("Error descr: $err_descr");
-		return;
+		return 0;
 	    }
 	    @bases = ("ou=DNS,$ldap_domain");
 	}
@@ -1928,9 +1994,11 @@ sub LdapPrepareToWrite {
 	    my $err = SCR->Read (".ldap.error") || {};
 	    my $err_descr = Dumper ($err);
 	    y2error ("Error descr: $err_descr");
-	    return;
+	    return 0;
 	}
     }
+
+    return 1;
 }
 
 BEGIN { $TYPEINFO{LdapStore} = ["function", "void" ]; }
